from telegrambot.test.factories.auth import AuthTokenFactory  # noqa
from telegrambot.test.factories.bot import BotFactory  # noqa
from telegrambot.test.factories.telegram_lib import (UserLibFactory, ChatLibFactory,  # noqa
                                                     MessageLibFactory, UpdateLibFactory)  # noqa
from telegrambot.test.factories.user import UserWebFactory  # noqa


from telegrambot.handlers.base import BotHandler

class RegexHandler(BotHandler):
    
    def __init__(self, pattern, callback):
        super(RegexHandler, self).__init__(callback)
        self.pattern = pattern
        
    def add_to_dispatcher(self, dispatcher):
        dispatcher.addTelegramRegexHandler(self.pattern, self.callback)
        
def regex(pattern, view):
    return RegexHandler(pattern, view)


from rest_framework.views import APIView
from telegrambot.serializers import UpdateSerializer
from telegrambot.models import Bot, AuthToken
from rest_framework.response import Response
from rest_framework import status
from telegram import Update
import logging
from django.views import generic
import sys
import traceback

logger = logging.getLogger(__name__)

class WebhookView(APIView):
    
    def post(self, request, token):
        serializer = UpdateSerializer(data=request.data)
        if serializer.is_valid():
            serializer.save()
            try:
                bot = Bot.objects.get(token=token)
                bot.process_update(Update.de_json(request.data))
            except Bot.DoesNotExist:
                logger.warning("Token %s not associated to a bot" % token)
                return Response(serializer.errors, status=status.HTTP_404_NOT_FOUND)
            except:
                exc_info = sys.exc_info()
                traceback.print_exception(*exc_info)
                logger.error("Error processing %s for token %s" % (request.data, token))
                return Response(serializer.errors, status=status.HTTP_500_INTERNAL_SERVER_ERROR)
            else:
                return Response(serializer.data, status=status.HTTP_200_OK)
        logger.error("Validation error: %s from message %s" % (serializer.errors, request.data))
        return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)
    
    
class AuthView(generic.TemplateView):
    template_name = 'telegrambot/authentication.html'
    
    def get_context_data(self, **kwargs):
        ctx = super(AuthView, self).get_context_data(**kwargs)
        ctx['bot'] = self.get_bot(self.kwargs['bot'])
        ctx['token'] = self.get_token(self.request.user)
        return ctx
    
    def get_bot(self, name):
        return Bot.objects.get(user_api__username=name)    
    
    def get_token(self, user):
        token, created = AuthToken.objects.get_or_create(user=user)
        if not created and token.expired():
            token.delete()
            token = AuthToken.objects.create(user=user)
        return token    


from telegrambot.handlers.base import BotHandler

class CommandHandler(BotHandler):
    
    def __init__(self, command, callback):
        super(CommandHandler, self).__init__(callback)
        self.command = command
        
    def add_to_dispatcher(self, dispatcher):
        dispatcher.addTelegramCommandHandler(self.command, self.callback)
        
def command(command, view):
    return CommandHandler(command, view)


from telegrambot.handlers.base import BotHandler

class MessageHandler(BotHandler):
    
    def add_to_dispatcher(self, dispatcher):
        dispatcher.addTelegramMessageHandler(self.callback)
        
def message(view):
    return MessageHandler(view)


class BotHandler(object):
    
    def __init__(self, callback):
        if not callable(callback):
            raise TypeError('view must be a callable')
        self.callback = callback
    
    def add_to_dispatcher(self, dispatcher):
        raise NotImplementedError


# -*- coding: utf-8 -*-
from django.db import models
from django.utils.encoding import python_2_unicode_compatible
from django.utils.translation import ugettext_lazy as _
from django.conf import settings
import importlib
from telegram import Updater
from django.db.models.signals import post_save
from django.dispatch import receiver
from django.core.urlresolvers import reverse
import logging
from telegrambot.models import User


logger = logging.getLogger(__file__)


@python_2_unicode_compatible
class Bot(models.Model):
    token = models.CharField(_('Token'), max_length=100, db_index=True)
    user_api = models.OneToOneField(User, verbose_name=_("Bot User"), related_name='bot', 
                                    on_delete=models.CASCADE, blank=True, null=True)
    ssl_certificate = models.FileField(_("SSL certificate"), upload_to='telegrambot/ssl/', 
                                       blank=True, null=True)
    enabled = models.BooleanField(_('Enable'), default=True)
    created = models.DateTimeField(('Date Created'), auto_now_add=True)
    modified = models.DateTimeField(_('Date Modified'), auto_now=True)    
    
    class Meta:
        verbose_name = _('Bot')
        verbose_name_plural = _('Bots')    
    
    def __init__(self, *args, **kwargs):
        super(Bot, self).__init__(*args, **kwargs)
        self.updater = None
        if self.token:
            self._configure_handlers()

    def _configure_handlers(self):
        # create handlers
        handlers_conf = getattr(settings, 'TELEGRAM_BOT_HANDLERS_CONF', None)
        if not handlers_conf:
            bothandlers = []
        else:
            bothandlers = importlib.import_module(handlers_conf).bothandlers
        self.updater = Updater(self.token)
        #  on different bot_handlers - answer in Telegram
        for handler in bothandlers:
            handler.add_to_dispatcher(self.updater.dispatcher)
            logger.debug("Handler %s added to bot %s" % (handler, str(self)))
            
    def __str__(self):
        return "%s" % (self.user_api.first_name or self.token if self.user_api else self.token)
            
    def process_update(self, update):
        """
        update: from python-telegram-bot
        """
        self.updater.dispatcher.processUpdate(update)
        

@receiver(post_save, sender=Bot)
def set_api(sender, instance, **kwargs):
    #  configure handlers if not yet
    if not instance.updater:
        instance._configure_handlers()
        logger.info("Success: Handlers for bot %s set" % str(instance))

    # set webhook
    url = None
    cert = None
    if instance.enabled:
        webhook = reverse('telegrambot:webhook', kwargs={'token': instance.token})        
        from django.contrib.sites.models import Site
        current_site = Site.objects.get_current()
        url = 'https://' + current_site.domain + webhook   
    if instance.ssl_certificate:
        cert = instance.ssl_certificate.open()
    instance.updater.bot.setWebhook(webhook_url=url, 
                                    certificate=cert)
    logger.info("Success: Webhook url %s for bot %s set" % (url, str(instance)))
    
    #  complete  Bot instance with api data
    if not instance.user_api:
        bot_api = instance.updater.bot.getMe()
        user_api, _ = User.objects.get_or_create(**bot_api.to_dict())
        instance.user_api = user_api
        instance.save()
        logger.info("Success: Bot api info for bot %s set" % str(instance))


from telegrambot.bot_views.generic import TemplateCommandView
from telegrambot.models import Bot

class LoginBotView(TemplateCommandView):
    template_text = "telegrambot/messages/login_required.txt"
    
    def generate_link(self, bot, link):
        
        from django.contrib.sites.models import Site
        current_site = Site.objects.get_current()
        return 'https://%s%s' % (current_site.domain, link)
    
    def get_bot(self, bot):
        return Bot.objects.get(token=bot.token)
    
    def get_context(self, bot, update, **kwargs):
    
        context = {'bot': self.get_bot(bot),
                   'link': self.generate_link(bot, kwargs['link'])}
        return context 


from telegrambot.handlers.command import command  # noqa
from telegrambot.handlers.message import message  # noqa
from telegrambot.handlers.regex import regex  # noqa
from telegrambot.handlers.unknown_command import unknown_command  # noqa


from telegrambot.handlers.base import BotHandler

class UnknownCommandHandler(BotHandler):
    
    def add_to_dispatcher(self, dispatcher):
        dispatcher.addUnknownTelegramCommandHandler(self.callback)
        
def unknown_command(view):
    return UnknownCommandHandler(view)